@@ -443,6 +443,15 @@ class DividendCalcMethod(ReadableEnum):
     BY_PER_UNIT = 2  # 每单位红利
 
 
+class FOFIncentiveFeeMode(ReadableEnum):
+    ''' 业绩计提方式 '''
+
+    SCST_HIGH_WL_S = 1  # 单人单笔高水位法（扣减份额）
+    SCST_HIGH_WL_N = 2  # 单人单笔高水位法（扣减净值）
+    INTE_HIGH_WL = 3  # 整体高水位法
+    INTE_HIGH_WL_WITH_SCST_R = 4  # 整体高水位配合单人单笔赎回时计提法
+
+
 class EMailParserType(ReadableEnum):
     ''' 电子邮件 parser 类型'''
 
