@@ -1022,6 +1022,18 @@ class DerivedDataApi(metaclass=Singleton):
             except Exception as e:
                 print(f'failed to get_hedge_fund_investor_div_carry <err_msg> {e} from {HedgeFundInvestorDivAndCarry.__tablename__}')
 
+    def get_hedge_fund_investor_div_carry_by_id(self, id: int):
+        with DerivedDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    HedgeFundInvestorDivAndCarry,
+                ).filter(
+                    HedgeFundInvestorDivAndCarry.id == id,
+                )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'failed to get_hedge_fund_investor_div_carry_by_id <err_msg> {e} from {HedgeFundInvestorDivAndCarry.__tablename__}')
+
     def delete_hedge_fund_investor_div_carry(self, id_to_delete: int) -> bool:
         with DerivedDatabaseConnector().managed_session() as session:
             try:
