@@ -253,6 +253,7 @@ class BasicDataPart1:
         df = df.drop(columns=['invest_type_class_1', 'invest_type_class_2']).reset_index()
         # 将场内变为场外的
         df['wind_id'] = df.wind_id.transform(lambda x: x.split('.')[0] + '.OF' if '.OF' not in x else x)
+        BasicDataApi().delete_fund_info(df.fund_id.to_list())
         self._data_helper._upload_basic(df, FundInfo.__table__.name)
 
     def index_info(self):
@@ -483,6 +484,7 @@ class BasicDataPart1:
                 return json.dumps({'-1': -1})
 
             normalized = x.BENCHMARK
+            normalized = normalized.lstrip('0')
             for k, v in _CHAR_TO_BE_REPLACED.items():
                 normalized = normalized.replace(k, v)
             for k, v in _CHAR_TO_BE_REPLACED_BY_RE.items():
@@ -567,17 +569,18 @@ class BasicDataPart1:
                 return json.dumps(replaced)
 
         r = self.fund_benchmark_parser(path)
-        all_funds_benchmark = pd.concat([r[0].BENCHMARK.rename('index_text'), r[1].rename('benchmark')], axis=1)
+        all_funds_benchmark = pd.concat([r[0].CODES.rename('em_id'), r[0].BENCHMARK.rename('index_text'), r[1].rename('benchmark')], axis=1)
         index_info = BasicDataApi().get_index_info()
         em_id_to_index_id = index_info.set_index('em_id')['index_id'].to_dict()
         all_funds_benchmark['benchmark_s_raw'] = all_funds_benchmark.apply(replaced_with_index_id, axis=1)
         now_fund_benchmark = BasicDataApi().get_fund_benchmark()
-        whole_funds_benchmark = all_funds_benchmark.join(now_fund_benchmark.set_index('em_id')[['assets', 'industry']], how='left')
+        whole_funds_benchmark = all_funds_benchmark.join(now_fund_benchmark.set_index('em_id')[['assets', 'industry']], on='em_id', how='left')
         whole_funds_benchmark['benchmark_s'] = whole_funds_benchmark.benchmark_s_raw
-        whole_funds_benchmark['fund_id'] = whole_funds_benchmark.apply(lambda x: x.name.split('.')[0] + '!0', axis=1)
-        # TODO asset and industry
-        BasicDataHelper()._upload_basic(whole_funds_benchmark.rename_axis(index='em_id').reset_index(), FundBenchmark.__table__.name)
+        whole_funds_benchmark['fund_id'] = whole_funds_benchmark.apply(lambda x: x.em_id.split('.')[0] + '!0', axis=1)
         # return whole_funds_benchmark
+        # TODO asset and industry
+        BasicDataApi().delete_fund_benchmark(whole_funds_benchmark.em_id.to_list())
+        BasicDataHelper()._upload_basic(whole_funds_benchmark, FundBenchmark.__table__.name)
 
     def index_price(self, start_date, end_date):
         try:
@@ -639,6 +642,7 @@ class BasicDataPart1:
                 #     if not df.empty:
                 #         index_id_list.append(index_id)
 
+            start_date_dt: datetime.date = pd.to_datetime(del_date, infer_datetime_format=True).date()
             end_date_dt: datetime.date = pd.to_datetime(end_date, infer_datetime_format=True).date()
             index_list = ['sp500']  #, 'dax30', 'n225']
             cm_index_list = ['sp500rmb']  #, 'dax30rmb', 'n225rmb']
@@ -653,10 +657,17 @@ class BasicDataPart1:
                 df['high'] *= cm_index_price['usd_central_parity_rate']
                 df['index_id'] = c
                 df['ret'] = df.close / df.close.shift(1) - 1
-                df = df.loc[raw_start_date_dt:end_date_dt, :]
-                df_list.append(df.reset_index())
+                df = df.loc[start_date_dt:end_date_dt, :]
+                df = df.reset_index()
+                df_list.append(df)
+                if not df.empty:
+                    index_id_list.append(c)
+
+            cx_index_df = cxindex_index_price[cxindex_index_price.datetime.between(start_date_dt, end_date_dt)]
+            df_list.append(cx_index_df)
+            if not cx_index_df.empty:
+                index_id_list.extend(list(cx_index_df.index_id.unique()))
 
-            df_list.append(cxindex_index_price[cxindex_index_price.datetime == end_date_dt])
             # 存进去美元计价的数据
             yahoo_index_price = yahoo_index_price[yahoo_index_price.index_id != 'n225']
 
@@ -666,7 +677,10 @@ class BasicDataPart1:
                 'ixic': 'nasdaq'
             }
             yahoo_index_price['index_id'] = yahoo_index_price.index_id.map(lambda x: naming_dict[x] if x in naming_dict else x)
-            df_list.append(yahoo_index_price[yahoo_index_price.datetime.between(raw_start_date_dt, end_date_dt)])
+            yahoo_index_df = yahoo_index_price[yahoo_index_price.datetime.between(start_date_dt, end_date_dt)]
+            df_list.append(yahoo_index_df)
+            if not yahoo_index_df.empty:
+                index_id_list.extend((yahoo_index_df.index_id.unique()))
 
             df = pd.concat(df_list, ignore_index=True)
             # TODO: 改成原子提交下边两个操作
@@ -1538,8 +1552,8 @@ class BasicDataPart1:
 
 if __name__ == '__main__':
     bdp = BasicDataPart1(BasicDataHelper())
-    # start_date = '20210601'
-    # end_date = '20210601'
+    # start_date = '20210624'
+    # end_date = '20210624'
     # bdp.fund_nav(start_date, end_date)
     # bdp.index_price(start_date, end_date)
     # bdp.rq_stock_price(start_date, end_date)
