@@ -40,7 +40,7 @@ class BasicFundRet(BasicDataPart1):
         # 获取区间内的交易日列表
         dts = BasicDataApi().get_trading_day_list(start_date, end_date).drop(columns='_update_time').datetime
         fund_list = self._fund_info_df.fund_id.tolist()
-        self.fund_nav = self.basic_api.get_fund_nav_with_date(self.start_date, self.end_date, fund_list)
+        self.fund_nav = self.basic_api.get_fund_nav_with_date(start_date=self.start_date, end_date=self.end_date, fund_list=fund_list)
         self.fund_nav = self.fund_nav.pivot_table(index='datetime', columns='fund_id', values='adjusted_net_value').reindex(dts).fillna(method='ffill')
         self.fund_nav_fill = self.fund_nav.fillna(method='bfill').copy()
         _fund_to_enddate_dict = self._fund_info_df[['fund_id', 'end_date']].set_index('fund_id').to_dict()['end_date']
@@ -132,4 +132,4 @@ class BasicFundRet(BasicDataPart1):
 
 if __name__ == '__main__':
     bfr = BasicFundRet(BasicDataHelper())
-    bfr.process_all('20200903')
+    bfr.process_all('20210601')
