@@ -100,9 +100,11 @@ class WebRawDataDownloader(object):
             # data from http://www.csindex.com.cn/zh-CN/indices/index-detail/H11073
             # File download url
             url = 'http://www.csindex.com.cn/uploads/file/autofile/perf/H11073perf.xls'
+            headers = {'User-Agent': ('Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_3) AppleWebKit/605.1.15 '
+                                      '(KHTML, like Gecko) Version/13.0.5 Safari/605.1.15')}
             filename = ''
             try:
-                r = self._session.get(url, timeout=(15, 60))
+                r = self._session.get(url, headers=headers, timeout=(15, 60))
                 filename = os.path.join('/tmp', 'H11073perf.xls')
                 with open(filename, 'wb') as f:
                     f.write(r.content)
@@ -259,3 +261,4 @@ if __name__ == "__main__":
     web_downloader = WebRawDataDownloader(data_helper)
     # web_downloader.cm_index_price('20200408', '20200508')
     # web_downloader.yahoo_index_price('20200427', '20200710')
+    # web_downloader.cxindex_index_price('20210617', '20210617')
