@@ -12,7 +12,7 @@ import pandas as pd
 from sqlalchemy.orm import sessionmaker
 
 from ...util.singleton import Singleton
-from ...constant import FOFTradeStatus, DividendCalcMethod, EMailParserType
+from ...constant import FOFTradeStatus, DividendCalcMethod, EMailParserType, FOFIncentiveFeeMode
 from ..view.derived_models import HedgeFundInvestorPurAndRedemp, HedgeFundInvestorDivAndCarry, HedgeFundCustodianData
 from ..wrapper.mysql import DerivedDatabaseConnector
 from ..api.basic import BasicDataApi
@@ -89,20 +89,24 @@ class HedgeFundDataManager(metaclass=Singleton):
             db_session.commit()
             db_session.close()
 
-    def _calc_share_details(self, fof_info, manager_id: str, fof_id: str, investor_id: str) -> List[Dict[str, Any]]:
+    def _calc_share_details(self, fof_info, manager_id: str, fof_id: str, investor_id: str, date: str) -> List[Dict[str, Any]]:
+        date = pd.to_datetime(date, infer_datetime_format=True).date()
+
         part1 = DerivedDataApi().get_hedge_fund_investor_pur_redemp(manager_id, [fof_id])
         assert part1 is not None, f'get pur redemp info failed (fof_id){fof_id} (manager_id){manager_id}'
-        part1 = part1.loc[part1.investor_id == investor_id, ['datetime', 'event_type', 'share_changed', 'net_asset_value', 'acc_unit_value']]
+        part1 = part1.loc[part1.investor_id == investor_id, ['datetime', 'event_type', 'share_changed', 'net_asset_value', 'acc_unit_value', 'water_line']]
 
         part2 = DerivedDataApi().get_hedge_fund_investor_div_carry(manager_id, [fof_id])
         assert part2 is not None, f'get div carry info failed (fof_id){fof_id} (manager_id){manager_id}'
-        part2 = part2.loc[(part2.investor_id == investor_id) & (part2.event_type.isin([FOFTradeStatus.DIVIDEND_VOLUME, FOFTradeStatus.DEDUCT_REWARD])), ['datetime', 'event_type', 'share_changed', 'net_asset_value', 'acc_unit_value']]
+        part2 = part2.loc[(part2.investor_id == investor_id) & (part2.event_type.isin([FOFTradeStatus.DIVIDEND_VOLUME, FOFTradeStatus.DEDUCT_REWARD])), ['datetime', 'event_type', 'share_changed', 'net_asset_value', 'acc_unit_value', 'water_line']]
 
         total = pd.concat([part1, part2]).sort_values(by='datetime')
+        total = total[total.datetime <= date]
         share_details = []
+        last_water_line = 1
         for row in total.itertuples(index=False):
             if row.event_type in (FOFTradeStatus.PURCHASE, FOFTradeStatus.SUBSCRIBE, FOFTradeStatus.DIVIDEND_VOLUME):
-                share_details.append({'datetime': row.datetime, 'share': row.share_changed, 'water_line': row.acc_unit_value})
+                share_details.append({'datetime': row.datetime, 'share': row.share_changed, 'water_line': row.water_line, 'acc_unit_value': row.acc_unit_value})
             elif row.event_type == FOFTradeStatus.REDEEM:
                 share_changed = abs(row.share_changed)
                 for one in share_details:
@@ -116,17 +120,24 @@ class HedgeFundDataManager(metaclass=Singleton):
                 else:
                     assert False, f"not enough shares to redeem in history (manager_id){manager_id} (fof_id){fof_id} (investor_id){investor_id} (left_shares){one['share']} (left_redemp){share_changed}"
             elif row.event_type == FOFTradeStatus.DEDUCT_REWARD:
-                for one in share_details:
-                    if math.isclose(one['share'], 0, rel_tol=1e-6):
-                        continue
-                    if one['acc_unit_value'] >= row.acc_unit_value:
-                        # 当前水位线未超过该笔份额的水位线 不需要计提业绩报酬
-                        continue
-                    carry = round(self._calc_carry(row.acc_unit_value, one['acc_unit_value'], fof_info.incentive_fee_type, fof_info.incentive_fee_str) * one['share'] / row.net_asset_value, 2)
-                    assert (one['share'] > carry) or math.isclose(one['share'], carry, rel_tol=1e-6), f"not enough shares to calc carry (manager_id){manager_id} (fof_id){fof_id} (investor_id){investor_id} (left_shares){one['share']} (carry){carry}"
-                    one['share'] -= carry
-                    # 水位线变成本次计提时的水位线了
-                    one['acc_unit_value'] = row.acc_unit_value
+                if fof_info.incentive_fee_mode in (FOFIncentiveFeeMode.SCST_HIGH_WL_S,):
+                    for one in share_details:
+                        if math.isclose(one['share'], 0, rel_tol=1e-6):
+                            continue
+                        if one['water_line'] >= row.acc_unit_value:
+                            # 当前水位线未超过该笔份额的水位线 不需要计提业绩报酬
+                            continue
+                        carry = round(self._calc_carry(row.acc_unit_value, one['water_line'], fof_info.incentive_fee_type, fof_info.incentive_fee_str) * one['share'] / row.net_asset_value, 2)
+                        assert (one['share'] > carry) or math.isclose(one['share'], carry, rel_tol=1e-6), f"not enough shares to calc carry (manager_id){manager_id} (fof_id){fof_id} (investor_id){investor_id} (left_shares){one['share']} (carry){carry}"
+                        one['share'] -= carry
+                        # 水位线变成本次计提时的水位线了
+                        one['water_line'] = row.acc_unit_value
+                elif fof_info.incentive_fee_mode in (FOFIncentiveFeeMode.INTE_HIGH_WL, FOFIncentiveFeeMode.INTE_HIGH_WL_WITH_SCST_R):
+                    assert row.water_line > last_water_line, '!!!'
+                    # profit = self._calc_carry(row.acc_unit_value, last_water_line, fof_info.incentive_fee_type, fof_info.incentive_fee_str)
+                else:
+                    assert False, f'do not support the fee mode {fof_info.incentive_fee_mode} temp (fof_id){fof_id} (manager_id){manager_id}'
+                last_water_line = row.water_line
             else:
                 assert False, f'invalid event type {row.event_type} during doing carry calc'
         return share_details
@@ -147,6 +158,7 @@ class HedgeFundDataManager(metaclass=Singleton):
                 investor_id : 投资人ID
                 net_asset_value : 单位净值
                 acc_unit_value : 累计净值
+                water_line : 水位线(一般是累计净值; 但整体高水位法下与累计净值可能不同)
                 amount : 申购金额/认购金额
                 raising_interest : 募集期利息
                 redemp_share : 赎回份额
@@ -177,19 +189,33 @@ class HedgeFundDataManager(metaclass=Singleton):
 
             # 计算业绩报酬计提
             carry_sum = 0
-            share_details = self._calc_share_details(fof_info, datas['manager_id'], datas['fof_id'], datas['investor_id'])
+            share_details = self._calc_share_details(fof_info, datas['manager_id'], datas['fof_id'], datas['investor_id'], datas['datetime'])
             for one in share_details:
                 if math.isclose(one['share'], 0, rel_tol=1e-6):
                     continue
                 if (one['share'] > datas['redemp_share']) or math.isclose(one['share'], datas['redemp_share'], rel_tol=1e-6):
                     one['share'] -= datas['redemp_share']
-                    if datas['acc_unit_value'] > one['water_line']:
-                        carry_sum += round(self._calc_carry(datas['acc_unit_value'], one['water_line'], fof_info.incentive_fee_type, fof_info.incentive_fee_str) * datas['redemp_share'], 2)
+                    if fof_info.incentive_fee_mode in (FOFIncentiveFeeMode.INTE_HIGH_WL_WITH_SCST_R,):
+                        if one['acc_unit_value'] < one['water_line']:
+                            # 如果这笔交易当时的交易净值在水下 需要在赎回时补扣业绩报酬
+                            profit = self._calc_carry(min(datas['acc_unit_value'], one['water_line']), one['acc_unit_value'], fof_info.incentive_fee_type, fof_info.incentive_fee_str)
+                            print(f"(profit){profit} (shares){datas['redemp_share']}")
+                            carry_sum += round(profit * datas['redemp_share'], 2)
+                    elif fof_info.incentive_fee_mode in (FOFIncentiveFeeMode.SCST_HIGH_WL_S,):
+                        if datas['acc_unit_value'] > one['water_line']:
+                            carry_sum += round(self._calc_carry(datas['acc_unit_value'], one['water_line'], fof_info.incentive_fee_type, fof_info.incentive_fee_str) * datas['redemp_share'], 2)
                     break
                 else:
                     datas['redemp_share'] -= one['share']
-                    if datas['acc_unit_value'] > one['water_line']:
-                        carry_sum += round(self._calc_carry(datas['acc_unit_value'], one['water_line'], fof_info.incentive_fee_type, fof_info.incentive_fee_str) * one['share'], 2)
+                    if fof_info.incentive_fee_mode in (FOFIncentiveFeeMode.INTE_HIGH_WL_WITH_SCST_R,):
+                        if one['acc_unit_value'] < one['water_line']:
+                            # 如果这笔交易当时的交易净值在水下 需要在赎回时补扣业绩报酬
+                            profit = self._calc_carry(min(datas['acc_unit_value'], one['water_line']), one['acc_unit_value'], fof_info.incentive_fee_type, fof_info.incentive_fee_str)
+                            print(f"(profit){profit} (shares){one['share']}")
+                            carry_sum += round(profit * one['share'], 2)
+                    elif fof_info.incentive_fee_mode in (FOFIncentiveFeeMode.SCST_HIGH_WL_S,):
+                        if datas['acc_unit_value'] > one['water_line']:
+                            carry_sum += round(self._calc_carry(datas['acc_unit_value'], one['water_line'], fof_info.incentive_fee_type, fof_info.incentive_fee_str) * one['share'], 2)
                     one['share'] = 0
             else:
                 assert False, f"not enough shares to redeem (fof_id){datas['fof_id']} (manager_id){datas['manager_id']} (left_shares){one['share']} (left_redemp){datas['redemp_share']}"
@@ -208,6 +234,8 @@ class HedgeFundDataManager(metaclass=Singleton):
 
         datas['applied_date'] = datas['datetime']
         datas['deposited_date'] = datas['datetime']
+        if 'water_line' not in datas:
+            datas['water_line'] = datas['acc_unit_value']
 
         s = pd.Series(datas).drop(['amount', 'redemp_share'], errors='ignore')
         # TODO: 原子操作下面几步
@@ -280,6 +308,7 @@ class HedgeFundDataManager(metaclass=Singleton):
                 investor_id : 投资人ID
                 net_asset_value : 单位净值
                 acc_unit_value : 累计净值
+                water_line : 水位线(一般是累计净值; 但整体高水位法下与累计净值可能不同)
                 total_dividend : 红利总额
                 cash_dividend : 现金分红金额
                 reinvest_amount : 再投资金额
@@ -321,7 +350,7 @@ class HedgeFundDataManager(metaclass=Singleton):
             for k in ('reinvest_amount', 'cash_dividend', 'total_dividend'):
                 datas[k] = float(datas[k])
 
-        for k in ('net_asset_value', 'acc_unit_value', 'carry_amount', 'share_changed', 'share_after_trans'):
+        for k in ('net_asset_value', 'acc_unit_value', 'water_line', 'carry_amount', 'share_changed', 'share_after_trans'):
             datas[k] = float(datas[k])
 
         datas['applied_date'] = datas['datetime']
@@ -425,6 +454,13 @@ class HedgeFundDataManager(metaclass=Singleton):
         df['manager_id'] = datas['manager_id']
         df['datetime'] = datas['datetime']
         df['event_type'] = datas['event_type']
+        columns_in_df = set(df.columns.array)
+        if 'net_asset_value' not in columns_in_df:
+            df['net_asset_value'] = datas['net_asset_value']
+        if 'acc_unit_value' not in columns_in_df:
+            df['acc_unit_value'] = datas['acc_unit_value']
+        if 'water_line' not in columns_in_df:
+            df['water_line'] = datas['acc_unit_value']
         return df.reset_index()
 
     def calc_carry_event(self, datas: Dict[str, Any]) -> pd.DataFrame:
@@ -451,41 +487,85 @@ class HedgeFundDataManager(metaclass=Singleton):
         assert fof_info.shape[0] == 1, f"invalid fof id {[datas['fof_id']]}"
         fof_info = fof_info.iloc[-1, :]
 
+        date = pd.to_datetime(datas['datetime'], infer_datetime_format=True).date()
+
         part1 = DerivedDataApi().get_hedge_fund_investor_pur_redemp(datas['manager_id'], [datas['fof_id']])
         assert part1 is not None, f"get pur redemp info failed (fof_id){datas['fof_id']} (manager_id){datas['manager_id']}"
+        part1 = part1[part1.datetime <= date]
 
         part2 = DerivedDataApi().get_hedge_fund_investor_div_carry(datas['manager_id'], [datas['fof_id']])
         assert part2 is not None, f"get pur div carry info failed (fof_id){datas['fof_id']} (manager_id){datas['manager_id']}"
+        part2 = part2[part2.datetime <= date]
 
-        total = pd.concat([part1, part2])
         carry_data = []
-        for investor_id in total.investor_id.unique():
-            share_details = self._calc_share_details(fof_info, datas['manager_id'], datas['fof_id'], investor_id)
-            share_before_trans = round(pd.DataFrame(share_details).share.sum(), 2)
-            carry_amount_sum = 0
-            share_changed = 0
-            for one in share_details:
-                if math.isclose(one['share'], 0, rel_tol=1e-6):
-                    continue
-                if one['water_line'] >= datas['acc_unit_value']:
-                    # 当前水位线未超过该笔份额的水位线 不需要计提业绩报酬
-                    continue
-                carry_amount = round(self._calc_carry(datas['acc_unit_value'], one['water_line'], fof_info.incentive_fee_type, fof_info.incentive_fee_str) * one['share'], 2)
-                carry = round(carry_amount / datas['net_asset_value'], 2)
-                assert (one['share'] > carry) or math.isclose(one['share'], carry, rel_tol=1e-6), f"not enough shares to calc carry (fof_id){datas['fof_id']} (manager_id){datas['manager_id']} (left_share){one['share']} (carry){carry}"
-                carry_amount_sum += carry_amount
-                share_changed -= carry
-                one['share'] -= carry
-                # 水位线变成本次计提时的水位线了
-                one['water_line'] = datas['acc_unit_value']
-            carry_data.append({
-                'investor_id': investor_id,
-                'datetime': datas['datetime'],
-                'share_before_trans': share_before_trans,
-                'carry_amount': carry_amount_sum,
-                'share_changed': share_changed,
-                'share_after_trans': share_before_trans + share_changed
-            })
+        total = pd.concat([part1, part2])
+        if fof_info.incentive_fee_mode in (FOFIncentiveFeeMode.INTE_HIGH_WL, FOFIncentiveFeeMode.INTE_HIGH_WL_WITH_SCST_R):
+            deduct_reward_info = part2[part2.event_type == FOFTradeStatus.DEDUCT_REWARD]
+            if deduct_reward_info.empty:
+                last_water_line = 1
+            else:
+                last_water_line = deduct_reward_info.sort_values(by='datetime').acc_unit_value.array[-1]
+            print(f"(fof_id){datas['fof_id']} (manager_id){datas['manager_id']} (last_water_line){last_water_line} (acc_unit_value){datas['acc_unit_value']}")
+            # 如果产品的水位线变高了 计提业绩报酬
+            assert datas['acc_unit_value'] > last_water_line, f"latest acc nav {last_water_line} is not greater than the last water line {datas['acc_unit_value']}, can not do carry calc (fof_id){datas['fof_id']} (manager_id){datas['manager_id']}"
+            profit = self._calc_carry(datas['acc_unit_value'], last_water_line, fof_info.incentive_fee_type, fof_info.incentive_fee_str)
+            for investor_id in total.investor_id.unique():
+                share_details = self._calc_share_details(fof_info, datas['manager_id'], datas['fof_id'], investor_id, datas['datetime'])
+                share_before_trans = round(pd.DataFrame(share_details).share.sum(), 2)
+                carry_amount_sum = 0
+                for one in share_details:
+                    if math.isclose(one['share'], 0, rel_tol=1e-6):
+                        continue
+                    if one['datetime'] == date:
+                        # 今天新买入的交易不做业绩计提
+                        continue
+                    carry_amount_sum += round(profit * one['share'], 2)
+                if not math.isclose(carry_amount_sum, 0):
+                    carry_data.append({
+                        'investor_id': investor_id,
+                        'datetime': datas['datetime'],
+                        'share_before_trans': share_before_trans,
+                        'carry_amount': carry_amount_sum,
+                        'share_changed': 0,
+                        'share_after_trans': share_before_trans,
+                        'net_asset_value': round(datas['net_asset_value'] - profit, fof_info.nav_decimals),
+                        'acc_unit_value': round(datas['acc_unit_value'] - profit, fof_info.nav_decimals),
+                        'water_line': round(datas['acc_unit_value'] - profit, fof_info.nav_decimals),
+                    })
+        elif fof_info.incentive_fee_mode in (FOFIncentiveFeeMode.SCST_HIGH_WL_S,):
+            for investor_id in total.investor_id.unique():
+                share_details = self._calc_share_details(fof_info, datas['manager_id'], datas['fof_id'], investor_id, datas['datetime'])
+                share_before_trans = round(pd.DataFrame(share_details).share.sum(), 2)
+                carry_amount_sum = 0
+                share_changed = 0
+                for one in share_details:
+                    if math.isclose(one['share'], 0, rel_tol=1e-6):
+                        continue
+                    if one['water_line'] >= datas['acc_unit_value']:
+                        # 当前水位线未超过该笔份额的水位线 不需要计提业绩报酬
+                        continue
+                    carry_amount = round(self._calc_carry(datas['acc_unit_value'], one['water_line'], fof_info.incentive_fee_type, fof_info.incentive_fee_str) * one['share'], 2)
+                    carry = round(carry_amount / datas['net_asset_value'], 2)
+                    assert (one['share'] > carry) or math.isclose(one['share'], carry, rel_tol=1e-6), f"not enough shares to calc carry (fof_id){datas['fof_id']} (manager_id){datas['manager_id']} (left_share){one['share']} (carry){carry}"
+                    carry_amount_sum += carry_amount
+                    share_changed -= carry
+                    one['share'] -= carry
+                    # 水位线变成本次计提时的水位线了
+                    one['water_line'] = datas['acc_unit_value']
+                carry_data.append({
+                    'investor_id': investor_id,
+                    'datetime': datas['datetime'],
+                    'share_before_trans': share_before_trans,
+                    'carry_amount': carry_amount_sum,
+                    'share_changed': share_changed,
+                    'share_after_trans': share_before_trans + share_changed,
+                    'net_asset_value': datas['net_asset_value'],
+                    'acc_unit_value': datas['acc_unit_value'],
+                    'water_line': datas['acc_unit_value'],
+                })
+        else:
+            assert False, f"do not support the fee mode {fof_info.incentive_fee_mode} temp (fof_id){datas['fof_id']} (manager_id){datas['manager_id']}"
+
         df = pd.DataFrame(carry_data)
         df['fof_id'] = datas['fof_id']
         df['manager_id'] = datas['manager_id']
