@@ -1,7 +1,10 @@
 
+from typing import Dict
+
 import numpy as np
+import pandas as pd
 
-from .factor_types import MomentumFactor
+from .factor_types import MomentumFactor, MixFactorBase
 from .basic_factors import StockPostPriceFactor
 from .utils import get_day
 
@@ -11,6 +14,7 @@ class AdjMomentumDateFactor(MomentumFactor):
     def __init__(self, num):
         super().__init__('m'+str(num)+'d')
         self.num = num
+        self._deps.add(StockPostPriceFactor())
 
     def calc(self):
         spp = StockPostPriceFactor().get()
@@ -44,15 +48,14 @@ class AdjMomentum12MFactor(AdjMomentumDateFactor):
         super().__init__(365)
 
 
-class MixMomentumFactor(MomentumFactor):
+class MixMomentumFactor(MixFactorBase, MomentumFactor):
     # 合成因子
     def __init__(self):
-        super().__init__('mixm')
-        self._universe: str = 'default'
+        super(MixFactorBase, self).__init__('mixm')
+        self._factor: Dict[pd.DataFrame] = {}
+        self._deps.add(AdjMomentum3MFactor())
+        self._deps.add(AdjMomentum6MFactor())
+        self._deps.add(AdjMomentum12MFactor())
 
-    # TODO: 这个接口写的不好，暂时先这样
-    def set_universe(self, universe: str):
-        self._universe = universe
-
-    def calc(self):
-        self._factor = AdjMomentum3MFactor().get_normalized(self._universe) + AdjMomentum6MFactor().get_normalized(self._universe) + AdjMomentum12MFactor().get_normalized(self._universe)
+    def calc(self, universe: str):
+        self._factor[universe] = AdjMomentum3MFactor().get_normalized(universe) + AdjMomentum6MFactor().get_normalized(universe) + AdjMomentum12MFactor().get_normalized(universe)
