@@ -10,15 +10,7 @@ from scipy.stats import gmean
 from functools import partial
 import statsmodels.api as sm
 from statsmodels.tsa.ar_model import AutoReg
-from ..data.api.raw import RawDataApi
-from ..data.api.basic import BasicDataApi
 from ..data.struct import TAAParam, TaaTunerParam, AssetWeight
-from ..data.api.derived import DerivedDataApi
-from ..fund.engine.asset_helper import TAAHelper, TAAStatusMode
-from ..data.wrapper.mysql import *
-from ..data.view.basic_models import *
-from ..data.view.derived_models import *
-from ..data.manager.manager_fund import DataManager
 from .calculator_item import CalculatorBase
 
 @dataclasses.dataclass
@@ -348,6 +340,7 @@ class Calculator:
                                       weight_list: pandas.core.series.Series):
         # fund_list 基金代码 比如 ['000017!0', '000020!0', '000031!0', '000051!0', '000061!0'] 形式
         # weight_list 权重代码 比如 [0.1,0.2,0.2,0.2,0.3] 形式
+        from ..data.manager.manager_fund import DataManager
         df = pandas.DataFrame({'fund_id':fund_list,'weight':weight_list}).set_index('fund_id')
         fund_info = DataManager.basic_data(func_name=f'get_fund_info', fund_list = fund_list.tolist())
         fund_type = 'stock'
@@ -408,6 +401,7 @@ class Calculator:
                                       weight_list: pandas.core.series.Series):
         # fund_list 基金代码 比如 ['000005!0', '000015!0', '000016!0', '000032!0', '000033!0']  array 形式
         # weight_list 权重代码 比如 [0.1,0.2,0.2,0.2,0.3] array 形式
+        from ..data.manager.manager_fund import DataManager
         df = pandas.DataFrame({'fund_id':fund_list,'weight':weight_list}).set_index('fund_id')
         fund_info = DataManager.basic_data(func_name=f'get_fund_info', fund_list = fund_list.tolist())
         fund_type = 'bond'
@@ -477,6 +471,7 @@ class Calculator:
 
     @staticmethod
     def get_fund_last_week_last_trade_df(dt:datetime.date=None, fund_list:list=[]):
+        from ..data.manager.manager_fund import DataManager
         # 日期不填默认上周最后交易日开始
         if dt is None:
             dt = DataManager.basic_data(func_name='get_last_week_last_trade_dt')
@@ -488,6 +483,7 @@ class Calculator:
 
     @staticmethod
     def get_fund_annual_ret_vol_recent_and_history(fund_list:list, hold_date_list:list):
+        from ..data.manager.manager_fund import DataManager
         nav = DataManager.basic_data(func_name='get_fund_nav_with_date',fund_list=fund_list)
         nav = nav.pivot_table(index='datetime',columns='fund_id',values='adjusted_net_value')
         result = {}
@@ -697,7 +693,8 @@ class Calculator:
         begin_date = datetime.date(2005,1,1)
         end_date = datetime.date(2020,1,1)
         Calculator.get_benchmark_result(index_list, weight_list, begin_date, end_date)
-        '''                   
+        '''     
+        from ..data.api.basic import BasicDataApi              
         basic = BasicDataApi()
         index_price = basic.get_index_price(index_list).pivot_table(index='datetime', columns='index_id', values='close')
         return Calculator.get_benchmark(index_price, weight_list, index_list, begin_date, end_date)
@@ -708,7 +705,7 @@ class Calculator:
                            windows:int=10*242,
                            min_periods:int=5*242,
                            pct_list:list=[0.15, 0.5, 0.6, 0.8]):
-        
+        from ..data.api.derived import DerivedDataApi
         assert (index_id in list(TaaTunerParam.PCT_PAIR.keys())) or (index_val is not None), \
             f'neither index_id{index_id} nor index_val {index_val} is valid'
 
@@ -749,6 +746,9 @@ class Calculator:
                        ):
         
         # load data
+        from ..data.api.basic import BasicDataApi
+        from ..data.api.derived import DerivedDataApi
+        from ..fund.engine.asset_helper import TAAHelper, TAAStatusMode
         _dts = BasicDataApi().get_trading_day_list()
         dts = _dts.datetime 
         index_val_data = DerivedDataApi().get_index_valuation_develop_without_date(index_ids=[index_id])
@@ -794,7 +794,8 @@ class Calculator:
 
     @staticmethod
     def get_index_level(dt:datetime.date=datetime.date(2020,7,20)):
-    
+        from ..data.wrapper.mysql import BasicDatabaseConnector, DerivedDatabaseConnector
+        from ..data.view.basic_models import IndexInfo, IndexValuationLongTerm
         with BasicDatabaseConnector().managed_session() as db_session:
             query = db_session.query(
                     IndexInfo
@@ -813,6 +814,8 @@ class Calculator:
 
     @staticmethod
     def get_index_level_index(index_id:str='hs300'):
+        from ..data.wrapper.mysql import BasicDatabaseConnector, DerivedDatabaseConnector
+        from ..data.view.basic_models import IndexInfo, IndexValuationLongTerm
         with BasicDatabaseConnector().managed_session() as db_session:
             query = db_session.query(
                     IndexInfo
@@ -853,6 +856,8 @@ class Calculator:
 
     @staticmethod
     def get_index_level_index_by_tag_method(index_id: str, tag_method: str):
+        from ..data.wrapper.mysql import DerivedDatabaseConnector
+        from ..data.view.derived_models import IndexValuationLongTerm
         val_dict = {
             'PB百分位': IndexValuationLongTerm.pb_pct,
             'PE百分位': IndexValuationLongTerm.pe_pct,
@@ -1049,7 +1054,9 @@ class Calculator:
     @staticmethod
     def get_index_stats(begin_date, end_date, time_para, index_list):
         try:
-            
+            from ..data.api.raw import RawDataApi
+            from ..data.api.basic import BasicDataApi
+            from ..data.api.derived import DerivedDataApi
             index_info = DerivedDataApi().get_index_val_info()
             res = []
             for i in index_info['index_group']:
@@ -1081,6 +1088,7 @@ class Calculator:
             }
             result = []
             #asset_info = BasicDataApi().get_asset_info_real(real_id_list=index_list)
+            
             begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
             basic = BasicDataApi()
             index_info = basic.get_index_info(index_list=index_list)
@@ -1123,6 +1131,9 @@ class Calculator:
             'roe':'ROE'
         }
         try:
+            from ..data.api.raw import RawDataApi
+            from ..data.api.basic import BasicDataApi
+            from ..data.api.derived import DerivedDataApi
             derived = DerivedDataApi()
             index_ids = [index_id]
             begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
@@ -1238,6 +1249,7 @@ class Calculator:
     @staticmethod
     def get_product_rolling_corr(fund_navs, windows=20, period='周度', index_id='hs300'):
         try:
+            from ..data.api.basic import BasicDataApi
             basic = BasicDataApi()
             try:
                 index_name = basic.get_index_info([index_id]).desc_name.tolist()[0]
@@ -1315,6 +1327,7 @@ class Calculator:
     @staticmethod
     def get_product_rolling_beta(fund_navs, windows=20, period='周度', index_id='hs300'):
         try:
+            from ..data.api.basic import BasicDataApi
             if fund_navs.empty:
                 df = pandas.DataFrame()
                 df.index.name = 'datetime'
@@ -1388,6 +1401,8 @@ class Calculator:
     @staticmethod
     def get_product_price_ratio_data(begin_date, end_date, time_para, index_id, fund_id):
         try:
+            from ..data.api.raw import RawDataApi
+            from ..data.api.basic import BasicDataApi
             raw = RawDataApi()
             basic = BasicDataApi()
             begin_date, end_date = raw.get_date_range(time_para, begin_date, end_date)
@@ -1452,6 +1467,8 @@ class Calculator:
     @staticmethod
     def get_product_price_ratio_data_calcu_part(begin_date, end_date, time_para, index_id, data_dic):
         try:
+            from ..data.api.raw import RawDataApi
+            from ..data.api.basic import BasicDataApi
             hf_fund_nav = data_dic['hf_fund_nav']
             hf_name_dic = data_dic['hf_name_dic']
             fund_id = hf_fund_nav.columns[0]
